@@ -1,5 +1,7 @@
 /* GnomENIUS Calculator
- * Copyright (C) 1997, 1998 George Lebl.
+ * Copyright (C) 1997, 1998 the Free Software Foundation.
+ *
+ * Author: George Lebl
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,12 +14,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, write to the  Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+ * USA.
  */
 
 #include <string.h>
-#include <gmp.h>
 #include <glib.h>
 #include <math.h>
 #include "calc.h"
@@ -35,6 +37,10 @@ extern calc_error_t error_num;
 
 /*my own power function for floats, very simple :) */
 static void mympf_pow_ui(mpf_t rop,mpf_t op,unsigned long int e);
+static void mympf_pow_z(mpf_t rop,mpf_t op,mpz_t e);
+
+/*my own power function for ints, very simple :) */
+static void mympz_pow_z(mpz_t rop,mpz_t op,mpz_t e);
 
 /*clear extra variables of type type, if type=op->type nothing is done*/
 static void mpwl_clear_extra_type(MpwRealNum *op,int type);
@@ -141,22 +147,93 @@ static char * mpwl_getstring(MpwRealNum * num, int max_digits,
 static void
 mympf_pow_ui(mpf_t rop,mpf_t op,unsigned long int e)
 {
-	mpf_t fr;
+	mpf_t answer;
+	mpf_t base;
 
-	if(rop==op) {
-		mpf_init(fr);
-		mpf_set_ui(fr,1);
-		for(;e>0;e--)
-			mpf_mul(fr,fr,op);
-		mpf_set(rop,fr);
-		mpf_clear(fr);
-	} else { /*different numbers safe to do a quicker way*/
+	if(e==0) {
 		mpf_set_ui(rop,1);
-		for(;e>0;e--)
-			mpf_mul(rop,rop,op);
+		return;
+	} else if(e==1) {
+		mpf_set(rop,op);
+		return;
 	}
+
+	mpf_init_set(base,op);
+	mpf_set_ui(rop,1);
+	while(e>0) {
+		if(e & 0x1) { /*if odd*/
+			mpf_mul(rop,rop,base);
+			e--;
+		} else { /* even */
+			mpf_mul(base,base,base);
+			e >>= 1; /*e /= 2*/
+		}
+	}
+	mpf_clear(base);
 }
 
+/*my own power function for floats, very simple :) */
+static void
+mympf_pow_z(mpf_t rop,mpf_t op,mpz_t e)
+{
+	unsigned long limb;
+	mpf_t answer;
+	mpf_t tmp;
+	mpz_t locale;
+
+	mpf_init_set_ui(answer,1);
+	mpf_init(tmp);
+	mpz_init_set(locale,e);
+
+	do {
+		limb=mpz_get_ui(locale);
+		mpz_tdiv_q_2exp(locale,locale,CHAR_BIT*sizeof(unsigned long));
+
+		if(limb==0)
+			continue;
+
+		mympf_pow_ui(tmp,op,limb);
+
+		mpf_mul(answer,answer,tmp);
+	} while(mpz_sgn(locale)!=0);
+
+	mpf_set(rop,answer);
+	mpf_clear(answer);
+	mpf_clear(tmp);
+	mpz_clear(locale);
+}
+
+
+/*my own power function for ints, very simple :) */
+static void
+mympz_pow_z(mpz_t rop,mpz_t op,mpz_t e)
+{
+	unsigned long limb;
+	mpz_t answer;
+	mpz_t tmp;
+	mpz_t locale;
+
+	mpz_init_set_ui(answer,1);
+	mpz_init(tmp);
+	mpz_init_set(locale,e);
+
+	do {
+		limb=mpz_get_ui(locale);
+		mpz_tdiv_q_2exp(locale,locale,CHAR_BIT*sizeof(unsigned long));
+
+		if(limb==0)
+			continue;
+
+		mpz_pow_ui(tmp,op,limb);
+
+		mpz_mul(answer,answer,tmp);
+	} while(mpz_sgn(locale)!=0);
+
+	mpz_set(rop,answer);
+	mpz_clear(answer);
+	mpz_clear(tmp);
+	mpz_clear(locale);
+}
 
 /*clear extra variables of type type, if type=op->type nothing is done*/
 static void
@@ -699,8 +776,9 @@ mpwl_pow_q(MpwRealNum *rop,MpwRealNum *op1,MpwRealNum *op2)
 	mpf_t fr;
 	mpf_t fr2;
 	mpf_t frt;
-	unsigned long ne;
-	unsigned long de;
+	unsigned long i;
+	mpf_t de;
+	mpz_t des;
 	int t;
 	int reverse=FALSE;
 
@@ -709,24 +787,20 @@ mpwl_pow_q(MpwRealNum *rop,MpwRealNum *op1,MpwRealNum *op2)
 		return FALSE;
 	}
 
-	if((mpz_cmp_ui(mpq_numref(op2->data.rval),ULONG_MAX)>0) ||
-		(mpz_cmp_ui(mpq_denref(op2->data.rval),ULONG_MAX)>0))
-	{
-		(*errorout)("Exponent numerator/quotient too big!");
-		error_num=NUMERICAL_MPW_ERROR;
-		return FALSE;
-	}
-
-	ne=mpz_get_ui(mpq_numref(op2->data.rval));
-	de=mpz_get_ui(mpq_denref(op2->data.rval));
-				/*this will make it allways
-					positive!*/
-	if(mpwl_sgn(op1)<0 && ((de/2)*2)==de) {
+	i=mpz_get_ui(mpq_denref(op2->data.rval));
+	if(mpwl_sgn(op1)<0 && !(i & 0x1)) {
 		/*it's gonna be complex*/
 		error_num=NUMERICAL_MPW_ERROR;
 		return TRUE;
 	}
 
+	mpz_init_set(des,mpq_denref(op2->data.rval));
+	mpz_sub_ui(des,des,1);
+
+	mpf_init(de);
+	mpf_set_z(de,mpq_denref(op2->data.rval));
+
+
 	if(mpq_sgn(op2->data.rval)<0)
 		reverse=TRUE;
 
@@ -744,11 +818,11 @@ mpwl_pow_q(MpwRealNum *rop,MpwRealNum *op1,MpwRealNum *op2)
 	mpf_div_ui(fr,op1->data.fval,2); /*use half the value
 					     as an initial guess*/
 	for(;;) {
-		mympf_pow_ui(fr2,fr,de);
+		mympf_pow_z(fr2,fr,mpq_denref(op2->data.rval));
 		mpf_sub(fr2,fr2,op1->data.fval);
 
-		mympf_pow_ui(frt,fr,de-1);
-		mpf_mul_ui(frt,frt,de);
+		mympf_pow_z(frt,fr,des);
+		mpf_mul(frt,frt,de);
 		mpf_div(fr2,fr2,frt);
 		mpf_neg(fr2,fr2);
 		mpf_add(fr2,fr2,fr);
@@ -760,10 +834,18 @@ mpwl_pow_q(MpwRealNum *rop,MpwRealNum *op1,MpwRealNum *op2)
 	}
 	mpf_clear(fr2);
 	mpf_clear(frt);
+	mpz_clear(des);
+	mpf_clear(de);
+
+	if(reverse) {
+		/*numerator will be negative*/
+		mpz_neg(mpq_numref(op2->data.rval),mpq_numref(op2->data.rval));
+		mympf_pow_z(fr,fr,mpq_numref(op2->data.rval));
+		mpz_neg(mpq_numref(op2->data.rval),mpq_numref(op2->data.rval));
 
-	mympf_pow_ui(fr,fr,ne);
-	if(reverse)
 		mpf_ui_div(fr,1,fr);
+	} else
+		mympf_pow_z(fr,fr,mpq_numref(op2->data.rval));
 
 	/*op1 might have equaled rop so clear extra type here*/
 	mpwl_clear_extra_type(op1,t);
@@ -784,9 +866,9 @@ mpwl_pow_ui(MpwRealNum *rop,MpwRealNum *op1,unsigned int e, int reverse)
 		case MPW_RATIONAL:
 			mpwl_init_type(&r,MPW_RATIONAL);
 			mpz_pow_ui(mpq_numref(r.data.rval),
-				op1->data.ival,e);
+				mpq_numref(op1->data.rval),e);
 			mpz_pow_ui(mpq_denref(r.data.rval),
-				op1->data.ival,e);
+				mpq_denref(op1->data.rval),e);
 			/*the exponent was negative! reverse the result!*/
 			if(reverse)
 				mpq_inv(r.data.rval,r.data.rval);
@@ -825,17 +907,66 @@ mpwl_pow_z(MpwRealNum *rop,MpwRealNum *op1,MpwRealNum *op2)
 	}
 
 	if(mpz_cmp_ui(op2->data.ival,ULONG_MAX)>0) {
-		(*errorout)("Exponent too big!");
-		error_num=NUMERICAL_MPW_ERROR;
-		return;
-	}
+		MpwRealNum r;
+		int reverse;
+		
+		if(mpz_sgn(op2->data.ival)<0) {
+			reverse = TRUE;
+			mpz_neg(op2->data.ival,op2->data.ival);
+		}
 
-	if(mpz_sgn(op2->data.ival)==0)
-		mpwl_set_ui(rop,1);
-	else if(mpz_sgn(op2->data.ival)>0)
-		mpwl_pow_ui(rop,op1,mpz_get_ui(op2->data.ival),FALSE);
-	else
-		mpwl_pow_ui(rop,op1,mpz_get_ui(op2->data.ival),TRUE);
+		switch(op1->type) {
+			case MPW_RATIONAL:
+				mpwl_init_type(&r,MPW_RATIONAL);
+				mympz_pow_z(mpq_numref(r.data.rval),
+					mpq_numref(op1->data.rval),
+					op2->data.ival);
+				mympz_pow_z(mpq_denref(r.data.rval),
+					mpq_denref(op1->data.rval),
+					op2->data.ival);
+				/*the exponent was negative! reverse the result!*/
+				if(reverse)
+					mpq_inv(r.data.rval,r.data.rval);
+				break;
+			case MPW_INTEGER:
+				if(!reverse) {
+					mpwl_init_type(&r,MPW_INTEGER);
+					mympz_pow_z(r.data.ival,
+						op1->data.ival,
+						op2->data.ival);
+				} else {
+					mpwl_init_type(&r,MPW_RATIONAL);
+					mympz_pow_z(mpq_denref(r.data.rval),
+						op1->data.ival,
+						op2->data.ival);
+					mympz_pow_z(mpq_numref(r.data.rval),
+						op1->data.ival,
+						op2->data.ival);
+				}
+				break;
+			case MPW_FLOAT:
+				mpwl_init_type(&r,MPW_FLOAT);
+				mympf_pow_z(r.data.fval,op1->data.fval,
+					op2->data.ival);
+
+				if(reverse)
+					mpf_ui_div(r.data.fval,1,r.data.fval);
+				break;
+		}
+		if(reverse)
+			mpz_neg(op2->data.ival,op2->data.ival);
+
+		mpwl_set(rop,&r);
+		mpwl_clear(&r);
+
+	} else {
+		if(mpz_sgn(op2->data.ival)==0)
+			mpwl_set_ui(rop,1);
+		else if(mpz_sgn(op2->data.ival)>0)
+			mpwl_pow_ui(rop,op1,mpz_get_ui(op2->data.ival),FALSE);
+		else
+			mpwl_pow_ui(rop,op1,mpz_get_ui(op2->data.ival),TRUE);
+	}
 }
 
 static int
@@ -1008,8 +1139,12 @@ str_format_float(char *p,long int e,int scientific_notation)
 	long int len;
 	int i;
 	if(((e-1)<-8 || (e-1)>8) || scientific_notation) {
-		p=my_realloc(p,strlen(p)+1,
-			strlen(p)+1+((int)log10(abs(e))+2)+1);
+		if(e!=0)
+			p=my_realloc(p,strlen(p)+1,
+				strlen(p)+1+((int)log10(abs(e))+2)+1);
+		else
+			p=my_realloc(p,strlen(p)+1,strlen(p)+3);
+			
 		if(p[0]=='-') {
 			if(strlen(p)>2) {
 				shiftstr(p+2,1);
@@ -1164,10 +1299,15 @@ mpwl_getstring(MpwRealNum * num, int max_digits,int scientific_notation,
 	return NULL;
 }
 
-#define mpw_uncomplex(rop)		\
-{					\
-	if(mpwl_sgn(rop->i)==0)		\
-		rop->type=MPW_REAL;	\
+#define mpw_uncomplex(rop)					\
+{								\
+	if(mpwl_sgn(rop->i)==0) {				\
+		rop->type=MPW_REAL;				\
+		if(rop->i->type!=MPW_INTEGER) {			\
+			mpwl_clear(rop->i);			\
+			mpwl_init_type(rop->i,MPW_INTEGER);	\
+		}						\
+	}							\
 }
 
 
@@ -1364,13 +1504,44 @@ mpw_div(mpw_ptr rop,mpw_ptr op1, mpw_ptr op2)
 		}
 		mpwl_div(rop->r,op1->r,op2->r);
 	} else {
-		if(mpwl_sgn(op2->r)==0 || mpwl_sgn(op2->i)==0) {
+		MpwRealNum t1;
+		MpwRealNum t2;
+		if(mpwl_sgn(op2->r)==0 && mpwl_sgn(op2->i)==0) {
 			(*errorout)("Division by zero!");
 			error_num=NUMERICAL_MPW_ERROR;
 			return;
 		}
-		/*FIXME: complex numbers*/
-		(*errorout)("div: can't deal with complex numbers");
+		rop->type = MPW_COMPLEX;
+
+		mpwl_init_type(&t1,MPW_INTEGER);
+		mpwl_init_type(&t2,MPW_INTEGER);
+
+		/*real part (r1r2 + i1i2)/(r2r2 + i2i2)*/
+		mpwl_mul(rop->r,op1->r,op2->r);
+		mpwl_mul(&t1,op1->i,op2->i);
+		mpwl_add(rop->r,rop->r,&t1);
+
+		mpwl_mul(&t1,op2->r,op2->r);
+		mpwl_mul(&t2,op2->i,op2->i);
+		mpwl_add(&t2,&t2,&t1);
+
+		mpwl_div(rop->r,rop->r,&t2);
+
+
+		/*imaginary part (i1r2 - r1i2)/(r2r2 + i2i2)*/
+		mpwl_mul(rop->i,op1->i,op2->r);
+		mpwl_mul(&t1,op1->r,op2->i);
+		mpwl_neg(&t1,&t1);
+		mpwl_add(rop->i,rop->i,&t1);
+
+		/*t2 is calculated above*/
+
+		mpwl_div(rop->i,rop->i,&t2);
+
+		mpwl_clear(&t1);
+		mpwl_clear(&t2);
+
+		mpw_uncomplex(rop);
 	}
 }
 
@@ -1411,13 +1582,40 @@ mpw_ui_div(mpw_ptr rop,unsigned int i,mpw_ptr op)
 		}
 		mpwl_ui_div(rop->r,i,op->r);
 	} else {
-		if(mpwl_sgn(op->r)==0 || mpwl_sgn(op->i)==0) {
+		MpwRealNum t1;
+		MpwRealNum t2;
+		if(mpwl_sgn(op->r)==0 && mpwl_sgn(op->i)==0) {
 			(*errorout)("Division by zero!");
 			error_num=NUMERICAL_MPW_ERROR;
 			return;
 		}
-		/*FIXME: complex numbers*/
-		(*errorout)("ui_div: can't deal with complex numbers");
+		rop->type = MPW_COMPLEX;
+
+		mpwl_init_type(&t1,MPW_INTEGER);
+		mpwl_init_type(&t2,MPW_INTEGER);
+
+		/*real part (r1r2)/(r2r2 + i2i2)*/
+		mpwl_mul_ui(rop->r,op->r,i);
+
+		mpwl_mul(&t1,op->r,op->r);
+		mpwl_mul(&t2,op->i,op->i);
+		mpwl_add(&t2,&t2,&t1);
+
+		mpwl_div(rop->r,rop->r,&t2);
+
+
+		/*imaginary part (- r1i2)/(r2r2 + i2i2)*/
+		mpwl_mul_ui(rop->i,op->i,i);
+		mpwl_neg(rop->i,rop->i);
+
+		/*t2 is calculated above*/
+
+		mpwl_div(rop->i,rop->i,&t2);
+
+		mpwl_clear(&t1);
+		mpwl_clear(&t2);
+
+		mpw_uncomplex(rop);
 	}
 }
 
@@ -1436,13 +1634,8 @@ mpw_mod(mpw_ptr rop,mpw_ptr op1, mpw_ptr op2)
 		}
 		mpwl_mod(rop->r,op1->r,op2->r);
 	} else {
-		if(mpwl_sgn(op2->r)==0 || mpwl_sgn(op2->i)==0) {
-			(*errorout)("Division by zero!");
-			error_num=NUMERICAL_MPW_ERROR;
-			return;
-		}
-		/*FIXME: complex numbers*/
-		(*errorout)("mod: can't deal with complex numbers");
+		error_num=NUMERICAL_MPW_ERROR;
+		(*errorout)("Can't modulo complex numbers");
 	}
 }
 
