@@ -42,7 +42,7 @@ proc_read_mem (ProcInfo *ps)
 
 	fscanf (f, "%*[^\n]\nMem: %u %u %u %u %u %u\n",
 		&ps->mem [PROC_MEM_TOTAL],
-		&u,
+		&ps->mem [PROC_MEM_USED],
 		&ps->mem [PROC_MEM_FREE],
 		&ps->mem [PROC_MEM_SHARED],
 		&ps->mem [PROC_MEM_BUF],
@@ -50,8 +50,8 @@ proc_read_mem (ProcInfo *ps)
 
 	ps->mem [PROC_MEM_USER] = ps->mem [PROC_MEM_TOTAL]
 		- ps->mem [PROC_MEM_FREE]
-		- ps->mem [PROC_MEM_SHARED]
-		- ps->mem [PROC_MEM_BUF];
+		- ps->mem [PROC_MEM_BUF]
+		- ps->mem [PROC_MEM_SHARED];
 
 	/* printf ("Mem: %u %u %u %u %u %u\n",
 		ps->mem [PROC_MEM_TOTAL],
