@@ -1,7 +1,11 @@
+#include <config.h>
+
 #include <string.h>
 #include <pwd.h>
 #include <sys/types.h>
 #include <sys/param.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 #include <asm/page.h>
 #include <linux/tasks.h>
 #include <time.h>
@@ -15,6 +19,7 @@
 #include "procbar.h"
 #include "proc.h"
 #include "details.h"
+#include "global.h"
 
 #include "asc.xpm"
 #include "dsc.xpm"
@@ -31,15 +36,22 @@ static GtkWidget *table;
 static GtkWidget *clist;
 static GtkWidget *clist_menu;
 static GtkWidget *vbox;
+static GtkWidget *handle_box;
 static GtkWidget *summary;
 
 static gint cfg_summary;
-static gint cfg_summary_save;
-static ProcBar *summary_cpu;
-static ProcBar *summary_mem;
-static ProcBar *summary_swap;
 static ProcInfo summary_info;
 
+gint cfg_show_summary;
+gint cfg_show_summary_cpu;
+gint cfg_show_summary_mem;
+gint cfg_show_summary_swap;
+gint cfg_has_swap;
+
+GtkWidget *summary_cpu;
+GtkWidget *summary_mem;
+GtkWidget *summary_swap = NULL;
+
 static GdkColor bar_cpu_colors [PROC_CPU_SIZE-1] = {
 	{0, 0xffff, 0xffff, 0x4fff},
 	{0, 0xdfff, 0xdfff, 0xdfff},
@@ -70,7 +82,6 @@ static uid_t user_uid;
 
 /* procview idle run or frozen */
 static gint cfg_run;
-static gint cfg_run_save;
 static gint run_tag = -1;
 static gint cfg_sf, cfg_order;
 
@@ -105,6 +116,12 @@ static void procview_init (gchar *, gchar *);
 static void procview_open_details ();
 static GtkWidget * procview_clist_prepare ();
 
+static GtkWidget     * renice_dialog = NULL;
+static GtkAdjustment * renice_aj;
+static GtkWidget     * renice_msg;
+
+static void procview_cfg_read ();
+
 #define CMP_DINT(NAME) \
 \
 static int cmp_ ## NAME (const ProcProcData **P, const ProcProcData **Q) \
@@ -148,6 +165,7 @@ CMP_DINT (pcpu)
 CMP_DINT (pmem)
 CMP_INT (size)
 CMP_INT (rss)
+CMP_INT (resident)
 CMP_INT (nice)
 CMP_INT (priority)
 CMP_INT (share)
@@ -157,34 +175,36 @@ CMP_STR (cmd)
 
 proc_field p_fields [] = {
 
-	{ "PID", "Process ID",
+	{ N_("PID"), N_("Process ID"),
 	  P_PID, cmp_pid, SORT_ASC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "User", "User name",
+	{ N_("User"), N_("User name"),
 	  P_USER, cmp_user, SORT_ASC, PROC_FILLUSR, GTK_JUSTIFY_LEFT, NULL},
-	{ "Pri", "Priority",
+	{ N_("Pri"), N_("Priority"),
 	  P_PRIORITY, cmp_priority, SORT_ASC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "Ni", "Nice",
+	{ N_("Ni"), N_("Nice"),
 	  P_NICE, cmp_nice, SORT_ASC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "Size", "Used memory size",
+	{ N_("Size"), N_("Used memory size"),
 	  P_SIZE, cmp_size, SORT_DSC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "RSS", "Resident memory size",
+	{ N_("RSS"), N_("Resident memory set size"),
 	  P_RSS, cmp_rss, SORT_DSC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "Share", "Shared memory size",
+	{ N_("Share"), N_("Shared memory size"),
 	  P_SHARE, cmp_share, SORT_DSC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "Stat", "Process state",
+	{ N_("Stat"), N_("Process state"),
 	  P_STATE, NULL, SORT_ASC, 0, GTK_JUSTIFY_LEFT, NULL},
-	{ "CPU", "Percent CPU usage",
+	{ N_("CPU"), N_("Percent CPU usage"),
 	  P_PCPU, cmp_pcpu, SORT_DSC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "MEM", "Percent memory usage",
+	{ N_("MEM"), N_("Percent memory usage"),
 	  P_PMEM, cmp_pmem, SORT_DSC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "Time", "Time consumed",
+	{ N_("Time"), N_("Time consumed"),
 	  P_TIME, cmp_time, SORT_DSC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "UTime", "User time consumed",
+	{ N_("UTime"), N_("User time consumed"),
 	  P_UTIME, cmp_utime, SORT_DSC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "STime", "System time consumed",
+	{ N_("STime"), N_("System time consumed"),
 	  P_STIME, cmp_stime, SORT_DSC, 0, GTK_JUSTIFY_RIGHT, NULL},
-	{ "Cmd", "Command",
+	{ N_("Cmd"), N_("Command"),
 	  P_CMD, cmp_cmd, SORT_ASC, PROC_FILLCMD, GTK_JUSTIFY_LEFT, NULL},
+	{ N_("Resident"), N_("Resident memory size"),
+	  P_RESIDENT, cmp_resident, SORT_DSC, 0, GTK_JUSTIFY_RIGHT, NULL},
 	{ NULL, NULL, 0, NULL, 0, 0, 0, NULL},
 };
 
@@ -260,12 +280,12 @@ procview_user_procs ()
 }
 
 void
-procview_start_stop ()
+procview_start_stop (gint start)
 {
-	GtkWidget *p = (cfg_run) ? xpm_stop : xpm_run;
+	GtkWidget *p = (!start) ? xpm_stop : xpm_run;
+	/* char **pd = (cfg_run) ? tb_timer_stopped_xpm : tb_timer_xpm; */
 
-	if (cfg_run) {
-		cfg_run = 0;
+	if (!start) {
 		if (run_tag != -1) {
 			gtk_timeout_remove (run_tag);
 			run_tag = -1;
@@ -273,7 +293,6 @@ procview_start_stop ()
 
 	} else {
 
-		cfg_run = 1;
 		if (run_tag != -1)
 			return;
 
@@ -281,11 +300,20 @@ procview_start_stop ()
 		run_tag = gtk_timeout_add (3000, procview_update, NULL);
 	}
 
-	GNOME_PIXMAP (TBC (RUN)->icon)->pixmap = GTK_PIXMAP (p)->pixmap,
-	GNOME_PIXMAP (TBC (RUN)->icon)->mask = GTK_PIXMAP (p)->mask,
+	gnome_stock_pixmap_widget_set_icon ((GnomeStockPixmapWidget *) (TBC (RUN)->icon),
+					    (start) ? GNOME_STOCK_PIXMAP_TIMER :
+					    GNOME_STOCK_PIXMAP_TIMER_STOP);
+
 	gtk_widget_queue_draw (GTK_WIDGET (TBC (RUN)->widget));
 }
 
+static void
+procview_tbar_time (GtkWidget *w, gpointer gp)
+{
+	cfg_run = !cfg_run;
+	procview_start_stop (cfg_run);
+}
+
 GnomeUIInfo procViewMenu [] = {
   {GNOME_APP_UI_ITEM, N_("All processes"), NULL, procview_all_procs, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
@@ -300,7 +328,7 @@ gint
 procview_cpu ()
 {
 	proc_read_cpu (&summary_info);
-	procbar_set_values (summary_cpu, summary_info.cpu);
+	gnome_proc_bar_set_values (GNOME_PROC_BAR (summary_cpu), summary_info.cpu);
 
 	return TRUE;
 }
@@ -309,8 +337,9 @@ gint
 procview_mem ()
 {
 	proc_read_mem (&summary_info);
-	procbar_set_values (summary_mem, summary_info.mem);
-	procbar_set_values (summary_swap, summary_info.swap);
+	gnome_proc_bar_set_values (GNOME_PROC_BAR (summary_mem), summary_info.mem);
+	if (summary_swap)
+		gnome_proc_bar_set_values (GNOME_PROC_BAR (summary_swap), summary_info.swap);
 
 	return TRUE;
 }
@@ -320,25 +349,36 @@ procview_summary ()
 {
 	GtkWidget *hbox;
 
-	hbox = gtk_hbox_new (TRUE, GNOME_PAD);
+	hbox = gtk_hbox_new (TRUE, GNOME_PAD_SMALL);
 
-	summary_cpu = procbar_new (xpm_cpu, PROC_CPU_SIZE-1, bar_cpu_colors,
+	proc_read_mem (&summary_info);
+	summary_cpu = gnome_proc_bar_new (xpm_cpu, PROC_CPU_SIZE-1, bar_cpu_colors,
 				   procview_cpu);
-	summary_mem = procbar_new (xpm_mem, PROC_MEM_SIZE-1, bar_mem_colors,
+	summary_mem = gnome_proc_bar_new (xpm_mem, PROC_MEM_SIZE-2, bar_mem_colors,
 				   procview_mem);
-	summary_swap = procbar_new (xpm_swap, PROC_SWAP_SIZE-1, bar_swap_colors,
-				    NULL);
+	cfg_has_swap = (summary_info.swap [PROC_SWAP_TOTAL]);
 
-	gtk_box_pack_start_defaults (GTK_BOX (hbox), summary_cpu->hbox);
-	gtk_box_pack_start_defaults (GTK_BOX (hbox), summary_mem->hbox);
-	gtk_box_pack_start_defaults (GTK_BOX (hbox), summary_swap->hbox);
+	summary_swap = gnome_proc_bar_new (xpm_swap, PROC_SWAP_SIZE-1, bar_swap_colors,
+					   NULL);
+
+	gtk_box_pack_start_defaults (GTK_BOX (hbox), summary_cpu);
+	gtk_box_pack_start_defaults (GTK_BOX (hbox), summary_mem);
+	if (cfg_has_swap)
+		gtk_box_pack_start_defaults (GTK_BOX (hbox), summary_swap);
+
+	if (cfg_show_summary_cpu)
+		gtk_widget_show (summary_cpu);
+
+	if (cfg_has_swap && cfg_show_summary_mem)
+		gtk_widget_show (summary_mem);
+
+	if (cfg_show_summary_swap && cfg_has_swap)
+		gtk_widget_show (summary_swap);
 
 	return hbox;
 }
 
-#define PIX(x) widget = gnome_create_pixmap_widget_d (GTK_WIDGET (window), \
-						      GTK_WIDGET (window), \
-						      x ## _xpm); \
+#define PIX(x) widget = gnome_pixmap_new_from_xpm_d (x ## _xpm); \
                         gtk_widget_ref (widget);
                
 
@@ -347,6 +387,8 @@ addProcessesView ()
 {
 	GtkWidget *widget;
 
+	procview_cfg_read ();
+
 	sort_asc = PIX (asc);
 	sort_dsc = PIX (dsc);
 
@@ -357,44 +399,39 @@ addProcessesView ()
 	xpm_mem = PIX (mem);
 	xpm_swap = PIX (swap);
 
-	cfg_run = !gnome_config_get_int ("/gtop/procview/run=1");
-	cfg_run_save = !cfg_run;
-
-	cfg_summary = !gnome_config_get_int ("/gtop/procview/summary=1");
-	cfg_summary_save = !cfg_summary;
-
-	cfg_sf = gnome_config_get_int ("/gtop/procview/sort_field=8");
-
-	cfg_order = gnome_config_get_int ("/gtop/procview/sort_order=1");
-
-	p_fields [cfg_sf].order = (cfg_order) ? SORT_DSC : SORT_ASC;
-
 	procview_init (gnome_config_get_string
 		       ("/gtop/procview/fields=0,1,2,3,4,5,6,7,8,9,10,13"),
 		       gnome_config_get_string
 		       ("/gtop/procview/geometry"
-			"=50,80,30,30,50,50,50,50,60,60,60,200"));
+			"=50,80,30,30,50,50,50,50,60,60,60,200"));	
 
-	nLabel = gtk_label_new (" Processes ");
+	nLabel = gtk_label_new (_("Processes"));
 	clist = procview_clist_prepare ();
 
 	procview_click_column (GTK_CLIST (clist), cfg_sf);
 
 	summary = procview_summary ();
-	vbox = gtk_vbox_new (FALSE, GNOME_PAD);
+	handle_box = gtk_handle_box_new ();
+	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
 
-	gtk_container_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
+	/* GTK_HANDLE_BOX (handle_box)->shrink_on_detach = 0; */
+#if 0
+	GTK_HANDLE_BOX (handle_box)->shadow_type = GTK_SHADOW_NONE;
+#endif
+	gtk_container_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
 
-	gtk_box_pack_start (GTK_BOX (vbox), summary, FALSE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (handle_box), summary);
+	gtk_box_pack_start (GTK_BOX (vbox), handle_box, FALSE, TRUE, 0);
 	gtk_box_pack_start_defaults (GTK_BOX (vbox), clist);
 
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, nLabel);
 
-	if (!cfg_summary)
+	if (cfg_show_summary)
 		gtk_widget_show (summary);
 
 	gtk_widget_show (nLabel);
 	gtk_widget_show (clist);
+	gtk_widget_show (handle_box);
 	gtk_widget_show (vbox);
 
 	/* pt = openproc (PROC_FILLUSR); */
@@ -402,20 +439,38 @@ addProcessesView ()
 	/* don't know why ?!? */
 	meminfo ();
 	/* procview_update (); */
-	procview_start_stop ();
 
 	return procview_handler;
 }
 
+static void
+procview_cfg_read ()
+{
+	printf ("proview_cfg_read\n");
+
+	cfg_run = gnome_config_get_int ("/gtop/procview/run=1");
+
+	cfg_show_summary = gnome_config_get_int ("/gtop/procview/show_summary=1");
+	cfg_show_summary_cpu  = gnome_config_get_int ("/gtop/procview/show_summary_cpu=1");
+	cfg_show_summary_mem  = gnome_config_get_int ("/gtop/procview/show_summary_mem=1");
+	cfg_show_summary_swap = gnome_config_get_int ("/gtop/procview/show_summary_swap=1");
+
+	cfg_sf    = gnome_config_get_int ("/gtop/procview/sort_field=8");
+	cfg_order = gnome_config_get_int ("/gtop/procview/sort_order=1");
+
+	p_fields [cfg_sf].order = (cfg_order) ? SORT_DSC : SORT_ASC;
+}
+
 static void
 procview_cfg_save ()
 {
 	/* printf ("proview_cfg_save\n"); */
 
 	gnome_config_set_int ("/gtop/procview/run", cfg_run);
-	gnome_config_set_int ("/gtop/procview/summary", !cfg_summary);
-	/*gnome_config_set_int ("/gtop/procview/sort_field", cfg_sf);*/
-	/*gnome_config_set_int ("/gtop/procview/sort_order", !cfg_order);*/
+	gnome_config_set_int ("/gtop/procview/show_summary", cfg_show_summary);
+	gnome_config_set_int ("/gtop/procview/show_summary_cpu", cfg_show_summary_cpu);
+	gnome_config_set_int ("/gtop/procview/show_summary_mem", cfg_show_summary_mem);
+	gnome_config_set_int ("/gtop/procview/show_summary_swap", cfg_show_summary_swap);
 
 	gnome_config_sync ();
 }
@@ -430,6 +485,11 @@ procview_handler (GtkNotebook *notebook, GtkNotebookPage *page, GTopPageSignal s
 
 		switch (s) {
 
+		case GTOP_PAGE_CFG_READ:
+
+			procview_cfg_read ();
+			break;
+
 		case GTOP_PAGE_CFG_SAVE:
 
 			procview_cfg_save ();
@@ -438,12 +498,14 @@ procview_handler (GtkNotebook *notebook, GtkNotebookPage *page, GTopPageSignal s
 		case GTOP_PAGE_ENTER:
 
 			/*printf ("procview page entered\n");*/
-			cfg_run = !cfg_run_save;
-			procview_start_stop ();
+			/* cfg_run = !cfg_run_save; */
+
+			gtop_time_cb = procview_tbar_time;
+			procview_start_stop (cfg_run);
 
 			if (!cfg_summary) {
-				procbar_start (summary_cpu, 200);
-				procbar_start (summary_mem, 200);
+				gnome_proc_bar_start (GNOME_PROC_BAR (summary_cpu), 200);
+				gnome_proc_bar_start (GNOME_PROC_BAR (summary_mem), 200);
 			}
 
 			gnome_app_menu_show (GNOME_APP(window),
@@ -454,13 +516,12 @@ procview_handler (GtkNotebook *notebook, GtkNotebookPage *page, GTopPageSignal s
 		case GTOP_PAGE_LEAVE:
 
 			/*printf ("procview page leaved\n");*/
-			cfg_run_save = cfg_run;
-			cfg_run = 1;
-			procview_start_stop ();
+			gtop_time_cb = NULL;
+			procview_start_stop (0);
 
 			if (!cfg_summary) {
-				procbar_stop (summary_cpu);
-				procbar_stop (summary_mem);
+				gnome_proc_bar_stop (GNOME_PROC_BAR (summary_cpu));
+				gnome_proc_bar_stop (GNOME_PROC_BAR (summary_mem));
 			}
 
 			gnome_app_menu_hide (GNOME_APP(window),
@@ -509,11 +570,13 @@ procview_click_column (GtkCList *cl, gint n)
 }
 
 static void
-procview_select_row (GtkCList *cl, gint row, gint col, GdkEventButton * e)
+procview_un_select_row (GtkCList *cl, gint row, gint col, GdkEventButton * e, gpointer gp)
 {
-	/*printf ("select_row: %d col: %d pid: %d %d\n", row, col, proc_tab [row]->pid);*/
+	gint i = (gint)gp;
+
+	/* printf ("select_row: %d col: %d pid: %d\n", row, col, proc_data [row]->p->pid); */
 	if (row < prev_count)
-		if (select_pid != proc_data [row]->p->pid || col == -1)
+		if (!i)
 			select_pid = proc_data [row]->p->pid;
 		else
 			select_pid = -1;
@@ -551,6 +614,97 @@ procview_open_details ()
 		procview_details (select_pid);
 }
 
+static void
+renice_destroy ()
+{
+	renice_dialog = NULL;
+}
+
+static void
+renice_cb (GtkWidget *w, gint n)
+{
+	if (n!=2) {
+		gint  rv, nv = renice_aj->value;
+		gchar *msg, tmp [256];
+
+		/* printf ("set priotiry of process %d to %d\n", select_pid, nv); */
+
+		rv = setpriority (PRIO_PROCESS, select_pid, nv);
+
+		if (rv == -1) {
+			switch (errno) {
+			case ESRCH:
+				g_snprintf (tmp, 256, _("Cannot locate process %d"), select_pid);
+				msg = tmp;
+				break;
+			case EACCES:
+				msg = _("Only root can set negative priority");
+				break;
+			case EPERM:
+				msg = _("Permission denied");
+				break;
+			}
+			n = 1;
+		} else {
+			procview_update ();
+			msg = "";
+		}
+
+		/* printf ("renice msg: %s\n", msg); */
+
+		gtk_label_set (GTK_LABEL (renice_msg), msg);
+	}
+
+	if (n==2 || !n)
+		gtk_widget_destroy (renice_dialog);
+}
+
+static void
+procview_renice ()
+{
+	if (!renice_dialog) {
+
+		GtkWidget *hs;
+		GtkWidget *lb;
+
+		renice_dialog = gnome_dialog_new (_("Renice"),
+						  GNOME_STOCK_BUTTON_OK,
+						  GNOME_STOCK_BUTTON_APPLY,
+						  GNOME_STOCK_BUTTON_CANCEL,
+						  NULL);
+		renice_aj = (GtkAdjustment *) gtk_adjustment_new (getpriority (PRIO_PROCESS,
+									       select_pid),
+						-20, 20, 1, 1, 0);
+		hs = gtk_hscale_new (renice_aj);
+		gtk_scale_set_digits (GTK_SCALE (hs), 0);
+		lb = gtk_label_new (_("Set priority"));
+		renice_msg = gtk_label_new ("");
+		gtk_misc_set_alignment (GTK_MISC (lb), 0, 1);
+		gtk_misc_set_alignment (GTK_MISC (renice_msg), 0, 1);
+
+		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (renice_dialog)->vbox),
+					     lb, TRUE, TRUE, 0);
+		gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (renice_dialog)->vbox),
+					     hs);
+		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (renice_dialog)->vbox),
+					     renice_msg, TRUE, TRUE, 0);
+
+		gnome_dialog_button_connect (GNOME_DIALOG (renice_dialog),
+					     0, (GtkSignalFunc) renice_cb, (gpointer) 0);
+		gnome_dialog_button_connect (GNOME_DIALOG (renice_dialog),
+					     1, (GtkSignalFunc) renice_cb, (gpointer) 1);
+		gnome_dialog_button_connect (GNOME_DIALOG (renice_dialog),
+					     2, (GtkSignalFunc) renice_cb, (gpointer) 2);
+		gtk_signal_connect (GTK_OBJECT (renice_dialog), "destroy",
+				    (GtkSignalFunc) renice_destroy, NULL);
+
+		gtk_widget_show (lb);
+		gtk_widget_show (hs);
+		gtk_widget_show (renice_msg);
+		gtk_widget_show (renice_dialog);
+	}
+}
+
 static GtkWidget *
 procview_clist_menu_prepare ()
 {
@@ -606,7 +760,7 @@ procview_clist_menu_prepare ()
 	mi = gtk_menu_item_new_with_label (_("SIGKILL"));
 	gtk_signal_connect (GTK_OBJECT (mi), "activate",
 			    GTK_SIGNAL_FUNC (procview_send_signal),
-			    (gpointer) SIGTERM);
+			    (gpointer) SIGKILL);
 	gtk_menu_append (GTK_MENU (menu), mi);
 	gtk_widget_show (mi);
 
@@ -615,6 +769,13 @@ procview_clist_menu_prepare ()
 	gtk_menu_append (GTK_MENU (menu), mi);
 	gtk_widget_show (mi);
 
+	mi = gtk_menu_item_new_with_label (_("Renice ..."));
+	gtk_signal_connect (GTK_OBJECT (mi), "activate",
+			    GTK_SIGNAL_FUNC (procview_renice),
+			    NULL);
+	gtk_menu_append (GTK_MENU (menu), mi);
+	gtk_widget_show (mi);
+
 	mi = gtk_menu_item_new_with_label (_("Details ..."));
 	gtk_signal_connect (GTK_OBJECT (mi), "activate",
 			    GTK_SIGNAL_FUNC (procview_open_details),
@@ -634,15 +795,21 @@ procview_clist_prepare ()
 	GList *g = geometry_list;
 	gchar **text = g_new (gchar *, field_list_length);
 	GtkWidget *l;
+	GtkTooltips *tips;
 
 	cl = gtk_clist_new (field_list_length);
 	gtk_clist_column_titles_show (GTK_CLIST (cl));
 	gtk_widget_set_name (cl, "ProcessList");
 
+	tips = gtk_tooltips_new ();
+	gtk_object_set_data (GTK_OBJECT (window), "tooltips", tips);
+
 	while (c) {
-		l = gtk_label_new (CF->label);
+		l = gtk_label_new (_(CF->label));
 		gtk_label_set_justify (GTK_LABEL (l), CF->justification);
 		gtk_widget_show (l);
+		gtk_tooltips_set_tip (tips, l,
+				      CF->long_info, "tip");
 		CF->hb = gtk_hbox_new (FALSE, 0);
 		gtk_box_pack_start_defaults (GTK_BOX (CF->hb), l);
 		gtk_widget_show (CF->hb);
@@ -659,7 +826,9 @@ procview_clist_prepare ()
 	gtk_signal_connect (GTK_OBJECT (cl), "click_column",
 			    GTK_SIGNAL_FUNC (procview_click_column), NULL);
 	gtk_signal_connect (GTK_OBJECT (cl), "select_row",
-			    GTK_SIGNAL_FUNC (procview_select_row), NULL);
+			    GTK_SIGNAL_FUNC (procview_un_select_row), (gpointer) 0);
+	gtk_signal_connect (GTK_OBJECT (cl), "unselect_row",
+			    GTK_SIGNAL_FUNC (procview_un_select_row), (gpointer) 1);
 	gtk_signal_connect (GTK_OBJECT (cl), "button_press_event",
 			    GTK_SIGNAL_FUNC (procview_clist_button_press), NULL);
         
@@ -676,6 +845,9 @@ procview_clist_prepare ()
 		g = g->next;
 	}
 
+	gtk_tooltips_set_tip (tips, cl,
+			      "tip", "tip");
+
 	return cl;
 }
 
@@ -688,7 +860,7 @@ procview_init (gchar *s, gchar *g)
 	gchar *gt;
 	gchar *cc=c;
 	gchar *gcc=gc;
-	gint index;
+	gint ndx;
 	gint *ip;
 
 	/*printf ("procview init: %s\n", s);*/
@@ -701,9 +873,9 @@ procview_init (gchar *s, gchar *g)
 		if (c && *c) {
 			if (t)
 				*t=0;
-			index = atoi (c);
-			/*printf ("%s %d: %s\n", s, index, p_fields [index].label);*/
-			field_list = g_list_append (field_list, p_fields+index);
+			ndx = atoi (c);
+			/*printf ("%s %d: %s\n", s, ndx, p_fields [ndx].label);*/
+			field_list = g_list_append (field_list, p_fields+ndx);
 			field_list_length++;
 			if (t)
 				c=t+1;
@@ -745,6 +917,7 @@ sprint_fmt (ProcProcData *d, p_fmt fmt)
 	case P_SHARE:
 	case P_PCPU:
 	case P_PMEM:
+	case P_RESIDENT:
 		switch (fmt) {
 		case P_TIME:
 			sprint_time (tmp, t->utime + t->stime);
@@ -779,6 +952,9 @@ sprint_fmt (ProcProcData *d, p_fmt fmt)
 		case P_PMEM:
 			sprintf (tmp, "%2d.%1d", d->pmem / 10, d->pmem % 10);
 			break;
+		case P_RESIDENT:
+			sprintf (tmp, "%d", t->resident << (PAGE_SHIFT - 10));
+			break;
 		}
 		rv = tmp;
 		break;
@@ -965,4 +1141,6 @@ procview_update ()
 
 	if (dwin)
 		procview_open_details ();
+
+	return TRUE;
 }
