@@ -1,3 +1,5 @@
+#include <config.h>
+
 #include "fsview.h"
 
 static GtkWidget *table;
@@ -29,7 +31,7 @@ sysview_handler (GtkNotebook *notebook, GtkNotebookPage *page, GTopPageSignal s)
 void *
 addSystemView () {
 	
-	nLabel = gtk_label_new (" System ");
+	nLabel = gtk_label_new (_("System"));
 	table =  gtk_table_new (10, 10, FALSE);
 
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, nLabel);
